﻿import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CoderResult;
import java.nio.charset.CodingErrorAction;
import java.util.Arrays;

public class MyTest {
	public static void main(String[] args) throws Exception {
		CharsetDecoder replaceDecoder = Charset.forName("UTF-8").newDecoder()
				.onMalformedInput(CodingErrorAction.REPLACE)
				.onUnmappableCharacter(CodingErrorAction.REPLACE);
		CharsetDecoder ignoreDecoder = Charset.forName("UTF-8").newDecoder()
				.onMalformedInput(CodingErrorAction.IGNORE)
				.onUnmappableCharacter(CodingErrorAction.IGNORE);

		String input = "你好";
		byte[] fullBytes = input.getBytes("UTF-8");
		System.out.println("input : " + input);
		System.out.print("input bytes in utf8 : ");
		print(fullBytes);
		System.out.println();
		System.out.println("=========================================");

		// decodeAndPrint(decoder, fullBytes);

		byte[] bytes0_4 = Arrays.copyOfRange(fullBytes, 0, 4);
		decodeAndPrint(replaceDecoder, bytes0_4, true);
		decodeAndPrint(replaceDecoder, bytes0_4, false);
		decodeAndPrint(ignoreDecoder, bytes0_4, true);
		decodeAndPrint(ignoreDecoder, bytes0_4, false);

		byte[] bytes0_5 = Arrays.copyOfRange(fullBytes, 0, 5);
		decodeAndPrint(replaceDecoder, bytes0_5, true);
		decodeAndPrint(replaceDecoder, bytes0_5, false);
		decodeAndPrint(ignoreDecoder, bytes0_5, true);
		decodeAndPrint(ignoreDecoder, bytes0_5, false);

	}
	
	private static void print(byte[] bytes) {
		for (byte b : bytes) {
			System.out.print(Integer.toString(b & 0xFF, 16) + " ");
			// System.out.print(b+" ");
		}
	}

	private static void decodeAndPrint(CharsetDecoder decoder, byte[] bytes,
			boolean complete) {
		decoder.reset();

		CharBuffer charBuffer = CharBuffer.allocate(bytes.length);
		ByteBuffer byteBuffer = ByteBuffer.wrap(bytes);

		CoderResult coderResult = decoder.decode(byteBuffer, charBuffer,
				complete);
		System.out.println("CoderResult: " + coderResult);

		System.out.println("input " + bytes.length + " bytes");
		System.out.println("decode " + byteBuffer.position() + " bytes");
		System.out.println("decode " + charBuffer.position() + " characters");

		System.out.println(Arrays.copyOf(charBuffer.array(), charBuffer
				.position()));
		System.out.println("-----------------------------------------------");
	}



}
